/* eslint-disable no-param-reassign */
import React from 'react';
import { Form, Select } from 'antd';
import { getNameFilterProps, getMixProps, debounce } from '../../../utils';

const FormItem = Form.Item;
const { Option: SelectOption } = Select;

/**
 * option数据结构参考: https://confluence.icsoc.net/pages/viewpage.action?pageId=21545602
 * 默认值放在options.defvalue上，上一次历史编辑值放在options.latestvalue中
 *
 * association自定义的customProps参考：
 * {
 *   type: 'association',
 *   props: { ... },
 *   nameFilter: {
 *     cle_id : {
 *       indivalue: { value: 'zs', text: '张三' }, // individual模式下，当前唯一选项
 *       action: function(...) { ... }, // 列表get接口
 *       mode: 'loadall' || 'realsearch' || 'individual' // 一次加载所有、实时搜索、固定一项
 *     }
 *   }
 * }
 */
class Association extends React.Component {
  constructor(props) {
    super(props);
    this.getDefault(props);
    this.state = {
      selectData: this.getInitSelectData(),
    };
    this.onSearch = debounce(this.onSearch, 500);
  }

  componentDidMount() {
    switch (this.mode) {
      // individual模式下，显示indivalue一项
      case 'individual': {
        const { form: { setFieldsValue }, obj: { name } } = this.props;
        const { indivalue } = this.getNameFilter(this.props);
        this.setState({ selectData: indivalue ? [indivalue] : [] }, () => {
          setFieldsValue({ [name]: (indivalue || {}).value });
        });
        // console.log('固定一项 ', indivalue);
        break;
      }
      default:
        break;
    }
  }

  componentWillReceiveProps(nextProps) {
    const { loadallData: thisLoadallData } = this.getNameFilter(this.props);
    const { loadallData: nextLoadallData } = this.getNameFilter(nextProps);
    switch (this.mode) {
      // loadall模式下
      // 拿到loadallData的值，放到this.state.selectData中
      case 'loadall': {
        const { selectData } = this.state;
        // TODO: 暂时只考虑loadallData从无到有的情况
        if (
          (!thisLoadallData || !thisLoadallData.length) &&
          nextLoadallData &&
          nextLoadallData.length
        ) {
          let newSelectData = []; // 新的selectData
          const values = []; // value字段数组，用来判断旧数据是否已存在于新数据
          newSelectData = nextLoadallData.map(v => {
            values.push(v.user_id || v.value);
            return {
              value: v.user_id || v.value,
              text: v.user_name || v.text,
            };
          });
          // 已有下拉项不存在于loadallData，则加入到loadallData
          selectData.forEach(v => {
            if (!values.includes(v.value)) {
              newSelectData.unshift(v);
            }
          });
          this.setState({ selectData: newSelectData });
        }
        break;
      }
      default:
        break;
    }
  }

  componentWillUnmount() {
    this.onSearch.clear();
  }

  // 拼接defvalue、latestvalue
  getInitSelectData = () => {
    const { obj: { options: { defvalue = [], latestvalue } } } = this.props;
    const copy = [...defvalue];
    if (latestvalue) {
      copy.unshift(latestvalue);
    }
    return copy;
  };

  /**
   * customProps.nameFilter[obj.name]
   * 拿到其中的action、mode、latestvalue字段，放在this上
   */
  getDefault = props => {
    const { action, mode } = this.getNameFilter(props);
    // this.latestvalue = latestvalue;
    this.action = action;
    this.mode = mode;
  };

  getNameFilter = props => {
    const { customProps, obj } = props;
    return getNameFilterProps(customProps, obj);
  };

  // 定义getFieldDecorator的配置参数
  getDecoratorProps = option => {
    const {
      customOptions: { noDefValue },
      obj: { options: { latestvalue = {} } },
    } = this.props;
    const { rules = [] } = option;
    return {
      rules: rules.map(rule => {
        if (rule.required && !rule.message) {
          rule.message = `${option.label}不能为空`;
        }
        return rule;
      }),
      initialValue: noDefValue
        ? latestvalue.value // noDefValue等于true时，不显示defvalue
        : (this.getInitSelectData()[0] || {}).value,
    };
  };

  // 组合customProps和类型modeProps，放到Select上
  getSelectProps = () => {
    const { customProps, obj } = this.props;
    const mixProps = getMixProps(customProps, obj);

    let modeProps = {};
    switch (this.mode) {
      // realsearch模式下，
      case 'loadall':
        modeProps = {
          optionFilterProp: 'children',
          filterOption: (input, option) =>
            option.props.children
              .toLowerCase()
              .includes(input.toLowerCase().trim()),
        };
        break;
      // realsearch模式下，关闭filterOption、自定义onSearch实时搜索
      // onChange时，下拉只保留当前项
      case 'realsearch':
        modeProps = {
          filterOption: false,
          defaultActiveFirstOption: false,
          onSearch: this.onSearch,
          onChange: this.onChange,
        };
        break;
      // individual模式下，开启不可编辑disabled
      case 'individual':
        modeProps = {
          disabled: true,
        };
        break;
      default:
        break;
    }

    return { ...modeProps, ...mixProps };
  };

  // 下拉选中
  onChange = value => {
    const { selectData } = this.state;
    this.setState({
      selectData: value ? [selectData.find(v => v.value === value)] : [],
    });
    // console.log('onChange ', value);
  };

  // 实时搜索
  onSearch = value => {
    const trimV = value.trim();
    if (trimV === '') {
      const { obj, form } = this.props;
      this.setState({
        selectData: [],
      });
      // console.log('空', form.getFieldValue(obj.name));
    } else {
      this.action(value.trim()).then(res => {
        const { payload: { data = [] } = {} } = res;
        const resData = data.data ? data.data.rows : data;
        this.setState({
          selectData: resData.map(v => ({
            value: v.cle_id || v.value,
            text: v.cle_name || v.text,
          })),
        });
        // console.log('res ', resData);
      });
    }
    // console.log('onSearch ', value);
  };

  render() {
    const {
      obj,
      form: { getFieldDecorator } = {},
      smallScreen = false,
      formItemLayout: layout = {},
    } = this.props;
    const { selectData } = this.state;

    const option = obj.options || {};
    const { readonly = false } = option;

    // const { dataSource } = this.getDataSource();
    const formItemLayout = smallScreen ? layout : {};
    return (
      <React.Fragment>
        <FormItem
          data-id={obj.id}
          label={
            <div className="inner-box" title={option.label || ''}>
              {option.label || ''}
            </div>
          }
          {...formItemLayout}
        >
          {getFieldDecorator(obj.name, this.getDecoratorProps(option))(
            <Select
              allowClear
              showArrow={false}
              showSearch
              size={option.size || 'default'}
              placeholder={option.placeholder || undefined}
              disabled={readonly}
              getPopupContainer={triggerNode => triggerNode.parentNode}
              {...this.getSelectProps()}
            >
              {selectData.length > 0 &&
                selectData.map(v => (
                  <SelectOption key={v.value} value={v.value}>
                    {v.text}
                  </SelectOption>
                ))}
            </Select>,
          )}
        </FormItem>
      </React.Fragment>
    );
  }
}

export default Association;
